"use strict";
/*
 * Module dependencies
 */
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var formatStats = require("webpack-format-messages");
var webpack_1 = require("../server/config/webpack");
var deferred_1 = require("../utils/deferred");
var routes_1 = require("../server/routes");
var MemoryFS = require("memory-fs");
var Webpack = require("webpack");
var pall_1 = require("../utils/pall");
var path_1 = require("path");
function Compile(params) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var root, env, aliases, fs, routes, s, b, configs, webpack, deferred, stats, files;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    root = params.root ? path_1.resolve(process.cwd(), params.root) : process.cwd();
                    env = params.environment || 'production';
                    aliases = params.aliases || {};
                    fs = new MemoryFS();
                    return [4 /*yield*/, routes_1.default(root)
                        // start loading the server-side config
                    ];
                case 1:
                    routes = _a.sent();
                    s = webpack_1.default({
                        environment: env,
                        target: 'server',
                        action: 'build',
                        aliases: aliases,
                        routes: routes,
                        root: root
                    });
                    b = webpack_1.default({
                        environment: env,
                        target: 'browser',
                        action: 'build',
                        aliases: aliases,
                        routes: routes,
                        root: root
                    });
                    return [4 /*yield*/, pall_1.default([s, b])];
                case 2:
                    configs = _a.sent();
                    if (configs instanceof Error)
                        return [2 /*return*/, configs
                            // build the configurations in parallel
                        ];
                    webpack = Webpack(configs);
                    webpack.outputFileSystem = fs;
                    deferred = new deferred_1.default();
                    webpack.run(function (err, stats) {
                        // TODO: turn into a custom error
                        if (err) {
                            if (err instanceof Error) {
                                return deferred.resolve(err);
                            }
                            // handle missing module and maybe more
                            var nerr = new Error(err.message || err);
                            nerr.stack = err.stack || '';
                            return deferred.resolve(nerr);
                        }
                        // TODO: turn into a custom error
                        var serr = checkError(stats);
                        if (serr) {
                            return deferred.resolve(serr);
                        }
                        return deferred.resolve(stats);
                    });
                    return [4 /*yield*/, deferred.wait()];
                case 3:
                    stats = _a.sent();
                    if (stats instanceof Error)
                        return [2 /*return*/, stats];
                    files = statsToFiles(fs, stats);
                    if (files instanceof Error)
                        return [2 /*return*/, files];
                    return [2 /*return*/, files];
            }
        });
    });
}
exports.default = Compile;
function checkError(stats) {
    var messages = formatStats(stats);
    if (messages.errors.length) {
        var errs = messages.errors.join('');
        return new Error("webpack: failed to compile.\n\n" + errs);
    }
    if (messages.warnings.length) {
        var warnings = messages.warnings.join('');
        return new Error("webpack: compiled with warnings.\n\n" + warnings);
    }
    return null;
}
function statsToFiles(fs, stats) {
    var json = stats.toJson({ assets: true, chunks: true });
    if (!json)
        return new Error('webkpack: no json emitted');
    var files = [];
    for (var i = 0; i < json.children.length; i++) {
        var child = json.children[i];
        var assets = child.assets;
        if (!Array.isArray(assets))
            continue;
        for (var j = 0; j < assets.length; j++) {
            var asset = assets[j];
            var path = asset.name;
            var contents = fs.readFileSync(path_1.resolve('/', path), 'utf8');
            files.push({ path: path, contents: contents });
        }
    }
    return files;
}
