"use strict";
/*
 * Module dependencies
 */
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var WebpackDevMiddleware = require("webpack-dev-middleware");
var WebpackHotMiddleware = require("webpack-hot-middleware");
var webpack_1 = require("../server/config/webpack");
var path_1 = require("path");
var routes_1 = require("../server/routes");
var Webpack = require("webpack");
var express = require("express");
var pall_1 = require("../utils/pall");
var http = require("http");
var messages_1 = require("../server/plugins/messages");
var isInteractive = !!process.stdout.isTTY;
var regexps = {};
function serve(params) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var root, env, aliases, app, routes, s, b, configs, webpack, dev, hot, server, serverClose;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0:
                    root = params.root ? path_1.resolve(process.cwd(), params.root) : process.cwd();
                    env = params.environment || 'development';
                    aliases = params.aliases || {};
                    app = express();
                    return [4 /*yield*/, routes_1.default(root)
                        // start loading the server-side config
                    ];
                case 1:
                    routes = _a.sent();
                    s = webpack_1.default({
                        environment: env,
                        target: 'server',
                        action: 'serve',
                        routes: routes,
                        aliases: aliases,
                        root: root
                    });
                    b = webpack_1.default({
                        environment: env,
                        target: 'browser',
                        action: 'serve',
                        routes: routes,
                        aliases: aliases,
                        root: root
                    });
                    return [4 /*yield*/, pall_1.default([s, b])];
                case 2:
                    configs = _a.sent();
                    if (configs instanceof Error)
                        return [2 /*return*/, configs
                            // build the configurations in parallel
                        ];
                    webpack = Webpack(configs);
                    // apply the plugin to the multicompiler
                    if (isInteractive) {
                        // right now hooks is not on the @types/webpack multicompiler
                        new messages_1.default().apply(webpack);
                    }
                    // support custom routing
                    if (routes.length) {
                        // rehydrate regexps
                        app.use(function (req, _res, next) {
                            if (req.method !== 'GET')
                                return next();
                            if (path_1.extname(req.path) !== '')
                                return next();
                            for (var i = 0, len = routes.length; i < len; i++) {
                                var route = routes[i];
                                var regexp = (regexps[route.regexp] =
                                    regexps[route.regexp] || new RegExp("" + route.regexp));
                                if (!regexp.test(req.path)) {
                                    continue;
                                }
                                req.url = route.path.replace(/\/+$/, '') + '/';
                                return next();
                            }
                            return next();
                        });
                    }
                    dev = WebpackDevMiddleware(webpack, {
                        publicPath: '/',
                        logLevel: 'silent'
                    });
                    app.use(dev);
                    hot = WebpackHotMiddleware(webpack, {
                        log: false,
                        path: '/__webpack_hmr',
                        heartbeat: 10 * 1000
                    });
                    app.use(hot);
                    // TODO: configurable
                    app.use(express.static(path_1.resolve(root, 'static')));
                    server = http.createServer(app);
                    serverClose = server.close;
                    // also close the middleware
                    server.close = function (fn) {
                        return serverClose.call(server, fn);
                    };
                    return [2 /*return*/, server];
            }
        });
    });
}
exports.default = serve;
