"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var preact_1 = require("preact");
var Document = /** @class */ (function (_super) {
    tslib_1.__extends(Document, _super);
    function Document() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Document.prototype.getChildContext = function () {
        return { _documentProps: this.props };
    };
    Document.prototype.render = function () {
        return (preact_1.h("html", null,
            preact_1.h(Head, null,
                preact_1.h("meta", { name: "viewport", content: "width=device-width, initial-scale=1" }),
                preact_1.h("link", { rel: "shortcut icon", type: "image/x-icon", href: "/static/favicon.ico" }),
                preact_1.h("meta", { name: "fragment", content: "!" })),
            preact_1.h("body", null,
                preact_1.h(Page, null),
                preact_1.h(Scripts, null))));
    };
    return Document;
}(preact_1.Component));
exports.default = Document;
var Head = /** @class */ (function (_super) {
    tslib_1.__extends(Head, _super);
    function Head() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Head.prototype.render = function () {
        var children = this.props.children;
        if (!children)
            return null;
        // prepend children if there's <Head />'s in the tree
        if (!this.context._documentProps)
            return preact_1.h("head", null, children);
        var _a = this.context._documentProps, head = _a.head, style = _a.style;
        // <style> tags inside the Document should get rendered properly
        walk(children, function (vnode) {
            if (vnode.nodeName === 'style' &&
                vnode.children &&
                vnode.children.length) {
                vnode.attributes = vnode.attributes || {};
                vnode.attributes.dangerouslySetInnerHTML = {
                    __html: vnode.children[0]
                };
            }
        });
        if (head) {
            children = (head || [])
                .concat(children)
                .filter(unique())
                .reverse();
        }
        if (style)
            children = (children || []).concat(style);
        return preact_1.h("head", null, children || []);
    };
    return Head;
}(preact_1.Component));
exports.Head = Head;
var Page = /** @class */ (function (_super) {
    tslib_1.__extends(Page, _super);
    function Page() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Page.prototype.render = function () {
        var page = this.context._documentProps.page;
        return preact_1.h("div", { id: "elmo.page", dangerouslySetInnerHTML: { __html: page } });
    };
    return Page;
}(preact_1.Component));
exports.Page = Page;
var Scripts = /** @class */ (function (_super) {
    tslib_1.__extends(Scripts, _super);
    function Scripts() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    Scripts.prototype.render = function () {
        var _a = this.context._documentProps, common = _a.common, script = _a.script;
        return (preact_1.h("div", { id: "elmo.scripts" },
            preact_1.h("script", { type: "text/javascript", dangerouslySetInnerHTML: { __html: 'window.prerenderReady=false;' } }),
            common,
            script));
    };
    return Scripts;
}(preact_1.Component));
exports.Scripts = Scripts;
var METATYPES = ['name', 'httpEquiv', 'charSet', 'itemProp'];
// returns a function for filtering head child elements
// which shouldn't be duplicated, like <title/>.
// TODO: typescript
function unique() {
    var tags = [];
    var metaTypes = [];
    var metaCategories = {};
    return function (h) {
        switch (h.nodeName) {
            case 'title':
            case 'base':
                if (~tags.indexOf(h.nodeName))
                    return false;
                tags.push(h.nodeName);
                break;
            case 'meta':
                for (var i = 0, len = METATYPES.length; i < len; i++) {
                    var metatype = METATYPES[i];
                    if (!h.attributes.hasOwnProperty(metatype))
                        continue;
                    if (metatype === 'charSet') {
                        if (~metaTypes.indexOf(metatype))
                            return false;
                        metaTypes.push(metatype);
                    }
                    else {
                        var category = h.attributes[metatype];
                        var categories = metaCategories[metatype] || [];
                        if (~categories.indexOf(category))
                            return false;
                        categories.push(category);
                        metaCategories[metatype] = categories;
                    }
                }
                break;
        }
        return true;
    };
}
function walk(vnodes, fn) {
    vnodes.forEach(function (vnode) {
        if (typeof vnode === 'object' && 'children' in vnode) {
            walk(vnode.children || [], fn);
            fn(vnode);
        }
    });
}
// Add in the static references
Document.Head = Head;
Document.Page = Page;
Document.Scripts = Scripts;
