"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var preact_1 = require("preact");
var DOMAttributeNames = {
    acceptCharset: 'accept-charset',
    className: 'class',
    htmlFor: 'for',
    httpEquiv: 'http-equiv'
};
var browser = typeof window !== 'undefined';
var mounted = [];
function reducer(components) {
    return components
        .map(function (c) { return c.children; })
        .filter(function (c) { return !!c; })
        .reduce(function (a, b) { return a.concat(b); }, [])
        .reverse()
        .filter(unique())
        .reverse()
        .map(function (v) {
        if (typeof v === 'string')
            return v;
        if (!('nodeName' in v))
            return v;
        var className = (v.attributes && v.attributes.className
            ? v.attributes.className + ' '
            : '') + 'preact-head';
        return preact_1.cloneElement(v, { className: className });
    });
}
function updateClient(head) {
    var tags = {};
    head.forEach(function (h) {
        // TODO: perhaps support custom components like <Meta />
        if (typeof h.nodeName !== 'string')
            return;
        var components = tags[h.nodeName] || [];
        components.push(h);
        tags[h.nodeName] = components;
    });
    updateTitle(tags.title ? tags.title[0] : null);
    var types = ['meta', 'base', 'link', 'style', 'script'];
    types.forEach(function (type) {
        updateElements(type, tags[type] || []);
    });
}
function updateElements(type, components) {
    var headEl = document.getElementsByTagName('head')[0];
    var oldTags = Array.prototype.slice.call(headEl.querySelectorAll(type + '.preact-head'));
    var newTags = components.map(VNodeToDOM).filter(function (newTag) {
        for (var i = 0, len = oldTags.length; i < len; i++) {
            var oldTag = oldTags[i];
            if (oldTag.isEqualNode(newTag)) {
                oldTags.splice(i, 1);
                return false;
            }
        }
        return true;
    });
    oldTags.forEach(function (t) { return t.parentNode && t.parentNode.removeChild(t); });
    newTags.forEach(function (t) { return headEl.appendChild(t); });
}
function VNodeToDOM(component) {
    var el = document.createElement(component.nodeName);
    var attrs = component.attributes || {};
    var children = component.children;
    for (var p in attrs) {
        if (!attrs.hasOwnProperty(p))
            continue;
        if (p === 'dangerouslySetInnerHTML')
            continue;
        var attr = DOMAttributeNames[p] || p.toLowerCase();
        el.setAttribute(attr, attrs[p]);
    }
    if (attrs['dangerouslySetInnerHTML']) {
        el.innerHTML = attrs['dangerouslySetInnerHTML'].__html || '';
    }
    else if (children) {
        el.textContent = typeof children === 'string' ? children : children.join('');
    }
    return el;
}
var METATYPES = ['name', 'httpEquiv', 'charSet', 'itemProp'];
// returns a function for filtering head child elements
// which shouldn't be duplicated, like <title/>.
// TODO: typescript
function unique() {
    var tags = [];
    var metaTypes = [];
    var metaCategories = {};
    return function (h) {
        switch (h.nodeName) {
            case 'title':
            case 'base':
                if (~tags.indexOf(h.nodeName))
                    return false;
                tags.push(h.nodeName);
                break;
            case 'meta':
                for (var i = 0, len = METATYPES.length; i < len; i++) {
                    var metatype = METATYPES[i];
                    if (!h.attributes.hasOwnProperty(metatype))
                        continue;
                    if (metatype === 'charSet') {
                        if (~metaTypes.indexOf(metatype))
                            return false;
                        metaTypes.push(metatype);
                    }
                    else {
                        var category = h.attributes[metatype];
                        var categories = metaCategories[metatype] || [];
                        if (~categories.indexOf(category))
                            return false;
                        categories.push(category);
                        metaCategories[metatype] = categories;
                    }
                }
                break;
        }
        return true;
    };
}
function updateTitle(component) {
    var title;
    if (component) {
        var children = component.children;
        title = typeof children === 'string' ? children : children.join('');
    }
    else {
        title = '';
    }
    if (title !== document.title) {
        document.title = title;
    }
}
function update() {
    if (!browser)
        return;
    var state = reducer(mounted.map(function (mount) { return mount.props; }));
    updateClient(state);
}
var Head = /** @class */ (function (_super) {
    tslib_1.__extends(Head, _super);
    function Head() {
        return _super !== null && _super.apply(this, arguments) || this;
    }
    // server: this should get called before rewind
    // client: doesn't matter where it is really
    Head.prototype.componentWillMount = function () {
        mounted.push(this);
        update();
    };
    Head.rewind = function () {
        var children = reducer(mounted.map(function (mount) { return mount.props; }));
        mounted = [];
        return children;
    };
    Head.prototype.componentDidUpdate = function () {
        update();
    };
    Head.prototype.componentWillUnmount = function () {
        var i = mounted.indexOf(this);
        if (~i)
            mounted.splice(i, 1);
        update();
    };
    Head.prototype.render = function () {
        return null;
    };
    return Head;
}(preact_1.Component));
exports.default = Head;
