"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var preact_1 = require("preact");
// deeply convert an XML DOM to VDOM
function compile(n, upgrader, options, index, arr) {
    if (options === void 0) { options = { type: 'html' }; }
    // handle text nodes
    if (n.nodeType === 3) {
        return compileText(n, index, arr);
    }
    // handle everything else
    if (n.nodeType !== 1)
        return null;
    var nodeName = String(n.nodeName).toLowerCase();
    // recursively compile the children
    var children = Array.prototype.map
        .call(n.childNodes, function (child, i, arr) {
        return compile(child, upgrader, options, i, arr);
    })
        .filter(function (child) { return child; });
    var component = preact_1.h(nodeName, getProps(n.attributes), children.length ? children : null);
    if (upgrader)
        return upgrade(component, upgrader);
    return component;
}
exports.default = compile;
// compile text nodes, trying to emulate HTML semantics by default, but these differ from JSX semantics
function compileText(n, index, arr) {
    if (arr === void 0) { arr = []; }
    var text = n.textContent !== null ? n.textContent : n.nodeValue || '';
    var isFirstOrLast = index === 0 || index === arr.length - 1;
    // trim strings but don't entirely collapse whitespace
    if (text.match(/^[\s\n]+$/g)) {
        text = ' ';
    }
    // skip leading/trailing whitespace
    if ((!text || text === ' ') && arr.length > 1 && isFirstOrLast)
        return null;
    return text;
}
function getProps(attrs) {
    var len = attrs && attrs.length;
    if (!len)
        return {};
    var props = {};
    for (var i = 0; i < len; i++) {
        var name = attrs[i].name;
        var value = attrs[i].value;
        props[name] = value === '' ? true : value;
    }
    return props;
}
/**
 * Upgrade our component to a virtual DOM node
 */
function upgrade(component, upgrader) {
    if (typeof component.nodeName === 'function')
        return component;
    var name = component.nodeName.toLowerCase();
    if (upgrader && upgrader.hasOwnProperty(name)) {
        var props = {};
        for (var k in component.attributes) {
            if (!component.attributes.hasOwnProperty(k))
                continue;
            props[k] = component.attributes[k];
        }
        props.key = component.key || undefined;
        props.children = component.children;
        var newNode = upgrader[name](props);
        if (!newNode)
            return null;
        component.nodeName = newNode.nodeName;
        component.attributes = newNode.attributes;
        component.children = newNode.children;
        component.key = newNode.key;
    }
    else {
        component.nodeName = name.replace(/[^a-z0-9-]/i, '');
    }
    return component;
}
