"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var htmlparser2 = require("htmlparser2");
var blocks = require("block-elements");
var inlines = require("inline-elements");
// import * as inv from 'invisibles'
var unified = require("unified");
var remarkParse = require("remark-parse");
var remark2rehype = require("remark-rehype");
var rehypeStringify = require("rehype-stringify");
// import * as toSlug from 'to-slug-case'
var domSerializer = require("dom-serializer");
// import * as preactHTML from 'preact-html'
// import * as preactRender from 'preact-render-to-string'
var prettier = require("prettier");
var buble = require("buble");
// implementation
function JS(code, options) {
    options = options || {};
    var resolve = options.resolve;
    var ast = parse(code);
    var requires = [['preact', 'preact']];
    walk(ast, function (node) {
        // all components must have path resolves
        if (resolve && isComponent(node)) {
            var p = resolve(node);
            if (!p)
                throw new Error("Couldn't find '" + node.tagName + "'.");
            requires.push([node.tagName, p]);
            return;
        }
        // optionally override regular HTML tags with paths
        if (resolve && isTag(node)) {
            var p = resolve(node);
            if (p)
                requires.push([node.tagName, p]);
            return;
        }
    });
    var html = renderHTML(ast);
    var js = buble.transform(html, { jsx: 'preact.h' }).code;
    var imports = requires
        .map(function (req) { return "import " + req[0] + " from \"" + req[1] + "\""; })
        .join('\n');
    var result = "\n" + imports + "\nexport default function () { return (" + js + ") }\n  ";
    var formatted = prettier.format(result, { semi: false });
    return formatted;
    // console.log(eval(result))
}
exports.JS = JS;
// function resolver(node: HTMLElement) {
// }
// function HTML(code: String, components) {
//   const ast = parse(code)
//   const html = renderHTML(ast)
//   const vnodes = preactHTML(html, components, { type: 'xml' })
//   return preactRender(vnodes[0])
// }
function parse(code) {
    // first render code blocks and inline
    // code because they may have HTML inside
    // of them.
    code = code.replace(/``?`?([^`]+)``?`?/g, escapeHtml);
    // get our HTML AST
    var hast = parseHTML(code);
    // walk our text nodes turning them into markdown
    walk(hast, function (node, parent) {
        if (!isText(node) || isCodeTag(parent))
            return;
        var text = node.value;
        // ignore empty tags
        if ((text || '').trim() === '')
            return;
        // parse the markdown
        var contents = processMarkdown(text);
        var subtree = parseHTML(contents);
        replace(node, subtree.children, parent);
    });
    // some sanitization and cleanup rules
    walk(hast, function (node, parent) {
        blockInsideInline(node, parent);
        collapseP(node, parent);
        collapsePInsideHeaders(node, parent);
    });
    return hast;
}
function processMarkdown(code) {
    return unified()
        .use(remarkParse)
        .use(remark2rehype)
        .use(rehypeStringify)
        .processSync(code).contents;
}
function parseHTML(code) {
    var ast = htmlparser2.parseDOM("<div class=\"mdx\">" + code + "</div>", {
        xmlMode: true
    })[0];
    // turn the AST into a proper HAST
    walk(ast, function (node) {
        var n = node;
        if (isHP2Node(node)) {
            delete node.next;
            delete node.prev;
            delete node.parent;
        }
        // handle tags
        if (isHP2Tag(n) && isTag(node)) {
            node.type = 'element';
            node.tagName = n.name;
            node.properties = n.attribs;
        }
        // handle text
        if (isHP2Text(n) && isText(node)) {
            // htmlparser2 adds this entity onto our
            // escaped HTML, we remove it here.
            node.value = n.data.replace(/#x26;/g, '');
        }
    });
    return ast;
}
function renderHTML(ast) {
    // turn the HAST back
    walk(ast, function (node) {
        var n = node;
        // handle tags
        if (isHP2Tag(node) && isTag(n)) {
            node.type = 'tag';
            node.name = n.tagName;
            node.attribs = n.properties;
        }
        // handle text
        if (isHP2Text(node) && isText(n)) {
            node.data = n.value;
        }
    });
    return domSerializer(ast, { xmlMode: true });
}
function isCodeTag(node) {
    return isTag(node) && node.tagName === 'code';
}
function isPTag(node) {
    return isTag(node) && node.tagName === 'p';
}
function isHeaderTag(node) {
    return isTag(node) && /^h[1-6]$/.test(node.tagName);
}
function isComponent(node) {
    return isTag(node) && node.tagName[0] === node.tagName[0].toUpperCase();
}
function isTag(node) {
    return !!(node && (node.type === 'element' || node.type === 'tag'));
}
function isText(node) {
    return !!(node && node.type === 'text');
}
function isHP2Node(node) {
    return (node.type === 'tag' || 'parent' in node || 'next' in node || 'prev' in node);
}
function isHP2Text(node) {
    return 'data' in node;
}
function isHP2Tag(node) {
    return node && (node.type === 'element' || node.type === 'tag');
}
function walk(node, fn, parent) {
    var children = node.children || [];
    for (var i = 0; i < children.length; i++) {
        walk(children[i], fn, node);
    }
    fn(node, parent);
}
function blockInsideInline(node, parent) {
    if (isBlock(node) && isInline(parent)) {
        unwrap(node, parent);
    }
}
function collapseP(node, parent) {
    if (isPTag(node) && isPTag(parent)) {
        unwrap(node, parent);
    }
}
function collapsePInsideHeaders(node, parent) {
    if (isPTag(node) && isHeaderTag(parent)) {
        unwrap(node, parent);
    }
}
// function unescapeCode(node?: HTMLNode, parent?: HTMLNode) {
//   if (!isText(node) || !isCodeTag(parent)) return
//   console.log('beforez', node.value)
//   node.value = node.value
//     .replace(/&quot;/g, '"')
//     .replace(/&amp;/g, '&')
//     .replace(/&#39;/g, "'")
//     .replace(/&lt;/g, '<')
//     .replace(/&gt;/g, '>')
//   console.log('afterz', node.value)
// }
function replace(node, replacements, parent) {
    if (!parent)
        return replacements;
    var children = parent.children || [];
    var i = children.indexOf(node);
    var newChildren = children
        .slice(0, i)
        .concat(replacements)
        .concat(children.slice(i + 1));
    parent.children = newChildren;
    return replacements;
}
function unwrap(node, parent) {
    return replace(node, node.children || [], parent);
}
function isBlock(node) {
    return isTag(node) && !!~blocks.indexOf(node.tagName.toLowerCase());
}
function isInline(node) {
    // console.log('inlines', inlines)
    return isTag(node) && !!~inlines.indexOf(node.tagName.toLowerCase());
}
function escapeHtml(string) {
    var str = '' + string;
    var match = /["'&<>]/.exec(str);
    if (!match) {
        return str;
    }
    var escape;
    var html = '';
    var index = 0;
    var lastIndex = 0;
    for (index = match.index; index < str.length; index++) {
        switch (str.charCodeAt(index)) {
            case 34: // "
                escape = '&quot;';
                break;
            case 38: // &
                escape = '&amp;';
                break;
            case 39: // '
                escape = '&#39;';
                break;
            case 60: // <
                escape = '&lt;';
                break;
            case 62: // >
                escape = '&gt;';
                break;
            default:
                continue;
        }
        if (lastIndex !== index) {
            html += str.substring(lastIndex, index);
        }
        lastIndex = index + 1;
        html += escape;
    }
    return lastIndex !== index ? html + str.substring(lastIndex, index) : html;
}
