"use strict";
//
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var static_1 = require("../plugins/static");
// import Watcher from '../plugins/watcher'
// import WatchFix from '../plugins/watch-fix'
var CopyWebpackPlugin = require("copy-webpack-plugin");
var readPkg = require("read-pkg-up");
// import * as MessagesPlugin from 'webpack-messages'
var rewrite_route_1 = require("../../utils/rewrite-route");
var resolves_1 = require("../../utils/resolves");
var glob_1 = require("../../utils/glob");
var path_1 = require("path");
var fs_1 = require("mz/fs");
// import Route from '../../lib/route'
var webpack = require("webpack");
// import Messages from '../plugins/messages'
/**
 * CWD
 */
var cwd = process.cwd();
/**
 * Environment Variables
 */
function Environment() {
    return Object.keys(process.env).reduce(function (o, k) {
        o['process.env.' + k] = JSON.stringify(process.env[k]);
        return o;
    }, {});
}
// NOTE: these are required server-side
// so the server will require them directly
// and know where they are.
// (e.g. require('elmo/router'))
function ServerAliases(projectRoot, elmoRoot) {
    // resolve the project's preact first
    // then look for elmo's preact if
    // we can't find it
    var preact = resolves_1.default(path_1.join(projectRoot, 'node_modules', 'preact'), path_1.join(elmoRoot, 'node_modules', 'preact'));
    if (!preact) {
        throw new Error('no preact...');
    }
    return {
        react: preact,
        'react-dom': preact,
        preact: preact,
        elmo: path_1.join(__dirname, '..', '..', 'elmo.js'),
        style: path_1.join(__dirname, '..', '..', 'lib', 'style', 'style.js'),
        tslib: path_1.join(elmoRoot, 'node_modules', 'tslib')
    };
}
// NOTE: these are required client-side
// (e.g. import Router from 'elmo/router')
// these should only be requires that are
// added to the build. the public-facing
// aliases are here to allow us to narrow
// down our elmo typescript loader
function BrowserAliases(projectRoot, elmoRoot) {
    // TODO: this should only work in development,
    // then have a helpful error when you go into
    // production.
    var preact = resolves_1.default(path_1.join(projectRoot, 'node_modules', 'preact'), path_1.join(elmoRoot, 'node_modules', 'preact'));
    // TODO: don't throw
    if (!preact) {
        throw new Error('no preact...');
    }
    return {
        preact: preact,
        react: preact,
        'react-dom': preact,
        unfetch: path_1.join(elmoRoot, 'node_modules', 'unfetch'),
        'promise-polyfill': path_1.join(elmoRoot, 'node_modules', 'promise-polyfill'),
        elmo: path_1.join(__dirname, '..', '..', 'elmo.js'),
        style: path_1.join(__dirname, '..', '..', 'lib', 'style', 'style.js'),
        tslib: path_1.join(elmoRoot, 'node_modules', 'tslib'),
        // TODO: clean this up:
        // these requirements are for the hot-middleware-client
        // and should only be pulled in during development
        // and when the action is serve
        'webpack-hot-middleware': path_1.join(elmoRoot, 'node_modules', 'webpack-hot-middleware'),
        'ansi-html': path_1.join(elmoRoot, 'node_modules', 'ansi-html'),
        'strip-ansi': path_1.join(elmoRoot, 'node_modules', 'strip-ansi'),
        'ansi-regex': path_1.join(elmoRoot, 'node_modules', 'ansi-regex'),
        'html-entities': path_1.join(elmoRoot, 'node_modules', 'html-entities')
    };
}
/**
 * Webpack Config
 */
function Webpack(input) {
    return tslib_1.__awaiter(this, void 0, void 0, function () {
        var pkgPath, ELMO_ROOT, target, environment, action, routes, aliases, root, serverAliases, browserAliases, config, pages, publicPages, rel, common, document_1, rules, plugins, constants;
        return tslib_1.__generator(this, function (_a) {
            switch (_a.label) {
                case 0: return [4 /*yield*/, readPkg({ cwd: __dirname })];
                case 1:
                    pkgPath = (_a.sent()).path;
                    ELMO_ROOT = path_1.dirname(pkgPath);
                    target = input.target, environment = input.environment, action = input.action, routes = input.routes, aliases = input.aliases;
                    root = path_1.resolve(process.cwd(), input.root || '.');
                    serverAliases = Object.assign(ServerAliases(root, ELMO_ROOT), aliases);
                    browserAliases = Object.assign(BrowserAliases(root, ELMO_ROOT), aliases);
                    config = {
                        mode: environment,
                        name: target,
                        context: root,
                        module: { rules: [] },
                        plugins: [],
                        entry: {}
                    };
                    return [4 /*yield*/, glob_1.default(root, path_1.join('pages', '**'))
                        // filter out any page that contains an underscore in
                        // any part of their path
                    ];
                case 2:
                    pages = _a.sent();
                    publicPages = pages.filter(function (page) {
                        var rel = path_1.relative(path_1.join(root, 'pages'), page);
                        var parts = rel.split(path_1.sep);
                        for (var i = 0; i < parts.length; i++) {
                            var part = parts[i];
                            if (part[0] === '_')
                                return false;
                        }
                        return true;
                    });
                    if (publicPages.length === 0) {
                        rel = path_1.relative(cwd, root);
                        return [2 /*return*/, new Error("no public pages found after searching: './" + rel + "/pages/**/*.{js,jsx,ts,tsx}'. Are you running from an elmo directory?")];
                    }
                    common = [];
                    // setup the entries
                    config.entry = publicPages.reduce(function (obj, page) {
                        var name = rewrite_route_1.default(root, target, page);
                        var bundle = [];
                        // add in the common bundles
                        bundle.push.apply(bundle, common);
                        // only include webpack hot middleware when we're
                        // not in production and using the dev server
                        // and we're building for the browser
                        if (action === 'serve' &&
                            environment !== 'production' &&
                            target === 'browser') {
                            bundle.push('webpack-hot-middleware/client?path=/__webpack_hmr&noInfo=true&name=' +
                                target);
                        }
                        obj[name] = bundle.concat(page);
                        return obj;
                    }, {});
                    // add in the document page if we're on the server
                    if (target === 'server') {
                        document_1 = pages.find(function (p) { return path_1.basename(p, path_1.extname(p)) === '_document'; });
                        config.entry['_document.html'] =
                            document_1 || path_1.join(__dirname, '../../lib/_document.js');
                    }
                    config.target = target === 'browser' ? 'web' : 'node';
                    if (target !== 'browser') {
                        // this is to ensure that we use node's actual require(...)
                        // which is needed for styled-jsx, but should probably be the
                        // default anyway.
                        config.externals = [
                            function (_parent, request, fn) {
                                if (serverAliases[request]) {
                                    return fn(null, serverAliases[request]);
                                }
                                else if (request[0] !== '.' && request[0] !== '/') {
                                    // externalize all node_modules
                                    return fn(null, request[0]);
                                }
                                else {
                                    // pass relative paths through (don't externalize)
                                    return fn(null, undefined);
                                }
                            }
                        ];
                    }
                    if (target === 'browser') {
                        config.node = {
                            console: false,
                            global: false,
                            process: false,
                            __filename: false,
                            __dirname: false,
                            Buffer: false,
                            setImmediate: false,
                            dns: false,
                            fs: false,
                            path: true,
                            url: false
                        };
                        // support electron renderer and
                        // web support seamlessly
                        config.externals = [
                            function (_parent, request, fn) {
                                if (request === 'electron') {
                                    return fn(null, request);
                                }
                                return fn(null, undefined);
                            }
                        ];
                    }
                    // resolves
                    config.resolve = {
                        extensions: ['.ts', '.js', '.tsx', '.jsx'],
                        modules: [path_1.join(root, 'node_modules')],
                        alias: browserAliases
                    };
                    // where to resolve loaders
                    config.resolveLoader = {
                        modules: [path_1.join(root, 'node_modules')]
                    };
                    /**
                     * Output
                     */
                    config.output = {
                        path: '/',
                        publicPath: '/',
                        // [name] ensures the path matches the entry key
                        filename: '[name]',
                        libraryTarget: 'umd'
                    };
                    rules = config.module.rules;
                    // support json files
                    rules.push({
                        test: /\.json$/,
                        use: ['json-loader']
                    });
                    // support loading .jsx files
                    rules.push({
                        test: /\.jsx/,
                        exclude: /(node_modules|bower_components)/,
                        loader: path_1.join(ELMO_ROOT, 'node_modules', 'babel-loader'),
                        options: {
                            plugins: [
                                // TODO: check that these plugins exist
                                path_1.join(ELMO_ROOT, 'node_modules', 'babel-plugin-transform-class-properties'),
                                [
                                    path_1.join(ELMO_ROOT, 'node_modules', 'babel-plugin-transform-react-jsx'),
                                    { pragma: 'h' }
                                ]
                            ]
                        }
                    });
                    // automatically add import { h } from 'preact' to jsx pages
                    // NOTE: we add this to tsx pages, but typescript will
                    // complain about not being able to resolve anyway, but
                    // it makes for simpler tests to include it
                    rules.push({
                        test: /\.[jt]sx/,
                        exclude: /(node_modules|bower_components)/,
                        loader: path_1.join(__dirname, '..', 'loaders', 'h.js')
                    });
                    // compile our project's typescript files
                    rules.push({
                        test: /\.tsx?$/,
                        exclude: /(node_modules|bower_components)/,
                        loader: path_1.join(__dirname, '..', 'loaders', 'typescript.js'),
                        query: { environment: environment, target: target, root: root }
                    });
                    // compile .mdx files
                    // rules.push({
                    //   test: /\.mdx$/,
                    //   include: publicPages.map(page => resolve(root, page)),
                    //   loader: join(ELMO_JS_SRC, 'server', 'loaders', 'mdx.js'),
                    //   query: { root }
                    // })
                    // if (target === 'browser') {
                    //   rules.push({
                    //     test: /\.mdx$/,
                    //     include: publicPages.map(page => resolve(root, page)),
                    //     loader: join(ELMO_JS_SRC, 'server', 'loaders', 'boot.js'),
                    //     query: { root }
                    //   })
                    // }
                    // certain typescript files are allowed to be compiled
                    // by the frontend configuration. allow certain
                    // modules to be compiled from elmo's ts source
                    // rules.push({
                    //   test: /\.tsx?$/,
                    //   include: [
                    //     join(ELMO_TS_SRC, 'elmo.ts'),
                    //     join(ELMO_TS_SRC, 'lib'),
                    //     join(ELMO_TS_SRC, 'client')
                    //   ],
                    //   loader: typescript_loader,
                    //   query: { environment, target, root }
                    // })
                    if (target === 'browser') {
                        // always polyfill to find bugs faster
                        // right now just polyfilling fetch and promise
                        rules.push({
                            test: /\.[tj]sx?$/,
                            include: publicPages.map(function (page) { return path_1.resolve(root, page); }),
                            loader: path_1.join(__dirname, '..', 'loaders', 'polyfills.js')
                        });
                        // boot script for client-side, only added to the top-level pages
                        rules.push({
                            test: /\.[jt]sx?$/,
                            include: publicPages.map(function (page) { return path_1.resolve(root, page); }),
                            use: {
                                loader: path_1.join(__dirname, '..', 'loaders', 'boot.js'),
                                query: { root: root, routes: routes }
                            }
                        });
                    }
                    /**
                     * Reverse plugins, webpack goes from bottom to top
                     * but top to bottom makes more sense in a config
                     */
                    rules = rules.reverse();
                    plugins = config.plugins || [];
                    constants = Environment();
                    // attach the environment to the NODE_ENV
                    constants['process.env.NODE_ENV'] = JSON.stringify(constants['process.env.NODE_ENV'] || environment);
                    // // add the routes into the router
                    // if (target === 'browser') {
                    //   constants = Object.assign(constants, {
                    //     _ELMO_ROUTES_: JSON.stringify(routes.length ? routes : await Routes(root))
                    //   })
                    // }
                    // Define environment variables
                    plugins.push(new webpack.DefinePlugin(constants));
                    // Don't write files if there are errors
                    plugins.push(new webpack.NoEmitOnErrorsPlugin());
                    // use names instead of IDs
                    plugins.push(new webpack.NamedModulesPlugin());
                    // fix watching
                    // server-side
                    if (target === 'server') {
                        plugins.push(new static_1.default({
                            elmoRoot: ELMO_ROOT,
                            routes: routes,
                            environment: environment,
                            target: target,
                            root: root,
                            action: action
                        }));
                    }
                    // setup minification & common chunks
                    if (target === 'browser') {
                        ;
                        config.optimization = {
                            minimize: environment === 'production',
                            splitChunks: {
                                name: 'common',
                                filename: 'common.js',
                                chunks: 'initial',
                                // TODO: these options should be optimized
                                // right now they make sure we always have
                                // a common.js, but sometimes it's not
                                // necessary.
                                minChunks: 1,
                                minSize: 0
                            }
                        };
                    }
                    if (!(target === 'server')) return [3 /*break*/, 5];
                    return [4 /*yield*/, fs_1.exists(path_1.join(root, 'static'))];
                case 3:
                    // CopyWebpackPlugin will produce an error if this directory doesn't exist already
                    // so we check it first
                    if (_a.sent()) {
                        plugins.push(new CopyWebpackPlugin([
                            {
                                context: root,
                                from: './static',
                                to: './static',
                                ignore: ['.*']
                            }
                        ]));
                    }
                    return [4 /*yield*/, fs_1.exists(path_1.join(root, '_redirects'))];
                case 4:
                    // copy over the _redirects
                    if (_a.sent()) {
                        plugins.push(new CopyWebpackPlugin([
                            {
                                context: root,
                                from: '_redirects',
                                to: '_redirects',
                                toType: 'file'
                            }
                        ], { copyUnmodified: true }));
                    }
                    _a.label = 5;
                case 5:
                    if (action === 'serve' && target === 'browser') {
                        // when serving, show friendly errors
                        // plugins.push(new MessagesPlugin())
                    }
                    // enable hot module replacement when in dev server in the browser
                    if (action === 'serve' &&
                        environment !== 'production' &&
                        target === 'browser') {
                        plugins.push(new webpack.HotModuleReplacementPlugin());
                    }
                    // minify scripts in production in the browser
                    if (environment === 'production' && target === 'browser') {
                        plugins.push(new webpack.LoaderOptionsPlugin({
                            minimize: true,
                            debug: false
                        }));
                    }
                    // config.devtool = environment === 'development' ? 'cheap-source-map' : false
                    config.devtool = false;
                    return [2 /*return*/, config];
            }
        });
    });
}
exports.default = Webpack;
