"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
// import rewrite from '../../utils/rewrite-route'
var preact_1 = require("preact");
var render_to_string = require("preact-render-to-string");
var evaluate_1 = require("../../utils/evaluate");
var path_1 = require("path");
var server_1 = require("../../lib/style/server");
/**
 * Module Exports
 */
var Static = /** @class */ (function () {
    function Static(options) {
        this.options = options;
    }
    Static.prototype.build = function (entries) {
        return tslib_1.__awaiter(this, void 0, void 0, function () {
            var elmoRoot, documentPath, Document, _loop_1, entry, state_1;
            return tslib_1.__generator(this, function (_a) {
                elmoRoot = this.options.elmoRoot;
                documentPath = Object.keys(entries).find(function (entry) {
                    return path_1.basename(entry, path_1.extname(entry)) === '_document';
                });
                if (!documentPath) {
                    return [2 /*return*/, new Error("couldn't find the _document in the entries")];
                }
                Document = evaluate_1.default(documentPath, entries[documentPath].source(), {
                    require: require
                });
                if (Document instanceof Error) {
                    return [2 /*return*/, Document];
                }
                else if (typeof Document !== 'function') {
                    return [2 /*return*/, new Error("document doesn't have a default export")];
                }
                // delete the document from the entries
                delete entries[documentPath];
                _loop_1 = function (entry) {
                    // ignore non-HTML files
                    if (path_1.extname(entry) !== '.html')
                        return "continue";
                    var Page = evaluate_1.default(entry, entries[entry].source(), { require: require });
                    if (Page instanceof Error) {
                        return { value: Page };
                    }
                    else if (typeof Page !== 'function') {
                        return { value: new Error("\"" + entry + "\" doesn't have a default export") };
                    }
                    var common = preact_1.h('script', {
                        type: 'text/javascript',
                        src: '/common.js'
                    });
                    var script = preact_1.h('script', {
                        type: 'text/javascript',
                        src: '/' + path_1.join(path_1.dirname(entry), 'index.js')
                    });
                    var props = {}; // TODO: support custom builds
                    // we render here, so we can rewind
                    var page = render_to_string(preact_1.h(Page, props));
                    var _a = rewindAll(elmoRoot), head = _a.head, style = _a.style;
                    // render the document into a string
                    var html = render_to_string(preact_1.h(Document, {
                        page: page,
                        common: common,
                        script: script,
                        head: head,
                        style: style,
                        props: props
                    }));
                    // update the entry
                    entries[entry] = {
                        source: function () { return html; },
                        size: function () { return html.length; }
                    };
                };
                // turn the node files into html files
                //
                // TODO: try workerize-ing this
                for (entry in entries) {
                    state_1 = _loop_1(entry);
                    if (typeof state_1 === "object")
                        return [2 /*return*/, state_1.value];
                }
                return [2 /*return*/];
            });
        });
    };
    Static.prototype.apply = function (compiler) {
        var self = this;
        compiler.hooks.thisCompilation.tap('static', function (compilation) {
            compilation.hooks.optimizeAssets.tapAsync('static', function (assets, done) {
                self
                    .build(assets)
                    .then(function (e) {
                    if (e)
                        compilation.errors.push(e);
                    done();
                })
                    .catch(function (e) {
                    done(e);
                });
            });
        });
    };
    return Static;
}());
exports.default = Static;
/**
 * Get the initial properties of the static build
 * // TODO: better inputs
 */
// function getBuildProps(Component: any): Promise<Object> {
//   if (Component.getBuildProps) {
//     return Component.getBuildProps({ build: true, path: '', query: {} })
//   } else {
//     return Promise.resolve({})
//   }
// }
/**
 * Render the HTML
 */
// TODO: not exactly sure how to type the Document when there's an evaluate()
// async function renderHTML(
//   Document: any,
//   params: {
//     elmoRoot: string
//     filepath: string
//     Component: any
//     props: any
//   }
// ) {
//   const Component = params.Component
//   const page = Stringify(<Component {...params.props} />)
//   const { head, style } = rewindAll(params.elmoRoot)
//   const docProps = await getBuildProps(Document)
//   // create a virtual DOM tree
//   const vhtml = (
//     <Document
//       page={page}
//       head={head}
//       style={style}
//       props={docProps || {}}
//       filepath={params.filepath}
//     />
//   )
//   const html = '<!DOCTYPE html>' + Stringify(vhtml)
//   // ignore any <style scoped> or <Head /> in the document
//   rewindAll(params.elmoRoot)
//   return html
// }
/**
 * Add the entry file
 */
// function addFile(entries: Entries, entry: string, html: string) {
//   const name = basename(dirname(entry))
//   entries[name + '/index.html'] = {
//     source: () => html,
//     size: () => html.length
//   }
//   return null
// }
// Okay so the reason for this is that when we call evaluate
// it may use a non-cached version of <Head /> or <Style/>
// depending on which module we require. These need to be
// pull from the cache to maintain their flush state.
// This function normalizes over all possible places
// you can get these modules from, so no matter where
// we require it we'll get the state.
function rewindAll(elmoRoot) {
    var dist = path_1.join(elmoRoot, 'dist');
    var src = path_1.join(elmoRoot, 'src');
    var cache = require.cache;
    var distHead = path_1.join(dist, 'lib', 'head.js');
    var srcHead = path_1.join(src, 'lib', 'head.ts');
    var distStyle = path_1.join(dist, 'lib', 'style', 'style.js');
    var srcStyle = path_1.join(src, 'lib', 'style', 'style.ts');
    var fn = null;
    var head = [];
    var style = [];
    for (var req in cache) {
        if (!cache[req])
            continue;
        switch (req) {
            case distHead:
                fn = cache[req].exports.default;
                head = head.concat(fn.rewind());
                break;
            case srcHead:
                fn = cache[req].exports.default;
                head = head.concat(fn.rewind());
                break;
            case distStyle:
                fn = cache[req].exports.default;
                style = style.concat(server_1.default(fn.flush()));
                break;
            case srcStyle:
                fn = cache[req].exports.default;
                style = style.concat(server_1.default(fn.flush()));
                break;
        }
    }
    return { head: head, style: style };
}
