"use strict";
Object.defineProperty(exports, "__esModule", { value: true });
var tslib_1 = require("tslib");
var path_1 = require("path");
// import * as loaderUtils from 'loader-utils'
var make_error_1 = require("make-error");
var ts = require("typescript");
var json5_1 = require("./json5");
// based on ts-node's: https://goo.gl/V7Qpnp
var defaultCompilerOptions = {
    sourceMap: true,
    inlineSourceMap: false,
    inlineSources: true,
    declaration: false,
    noEmit: false
};
/**
 * Transpile function
 */
function transpile(input) {
    var root = input.root, path = input.path, source = input.source, target = input.target, transformers = input.transformers;
    // read the tsconfig via json5
    var tsconfig = json5_1.default(path_1.join(root, 'tsconfig.json'));
    // use provided config or ts-node's defaults
    var compilerOptions = tsconfig.compilerOptions
        ? tsconfig.compilerOptions || {}
        : defaultCompilerOptions;
    // default module support
    compilerOptions.module =
        target === 'server' ? ts.ModuleKind.CommonJS : ts.ModuleKind.ESNext;
    // add in jsx compilation when the filepath is tsx
    if (path_1.extname(path) === '.tsx') {
        compilerOptions = Object.assign(compilerOptions, {
            jsx: 'react',
            jsxFactory: 'h'
        });
    }
    var result = ts.transpileModule(source, {
        fileName: path,
        reportDiagnostics: true,
        compilerOptions: compilerOptions,
        transformers: transformers || {}
    });
    // parse an error if there is one
    if (result.diagnostics && result.diagnostics.length) {
        return new (TSError.bind.apply(TSError, [void 0].concat(formatDiagnostics(result.diagnostics, root, 0))))();
    }
    return result;
}
exports.default = transpile;
/**
 * TypeScript diagnostics error.
 */
var TSError = /** @class */ (function (_super) {
    tslib_1.__extends(TSError, _super);
    function TSError() {
        var diagnostics = [];
        for (var _i = 0; _i < arguments.length; _i++) {
            diagnostics[_i] = arguments[_i];
        }
        var _this = _super.call(this, "\u2A2F Unable to compile TypeScript\n" + diagnostics
            .map(function (x) { return x.message; })
            .join('\n')) || this;
        _this.name = 'TSError';
        return _this;
    }
    return TSError;
}(make_error_1.BaseError));
function formatDiagnostics(diagnostics, root, lineOffset) {
    if (diagnostics === void 0) { diagnostics = []; }
    return diagnostics.map(function (x) { return formatDiagnostic(x, root, lineOffset); });
}
// create an error if it exists
function formatDiagnostic(diagnostic, root, lineOffset) {
    var messageText = ts.flattenDiagnosticMessageText(diagnostic.messageText, '\n');
    var code = diagnostic.code;
    if (!diagnostic.file) {
        return { message: messageText + " (" + code + ")", code: code };
    }
    var path = path_1.relative(root, diagnostic.file.fileName);
    if (!diagnostic.start) {
        return { message: path + ": " + messageText + " (" + code + ")", code: code };
    }
    var _a = diagnostic.file.getLineAndCharacterOfPosition(diagnostic.start), line = _a.line, character = _a.character;
    var message = path + " (" + (line + 1 + lineOffset) + "," + (character +
        1) + "): " + messageText + " (" + code + ")";
    return { message: message, code: code };
}
