#!/usr/bin/env node

import Analyze from '../commands/analyze'
import Serve from '../commands/serve'
import Build from '../commands/build'
import Setup from '../commands/setup'
// import Watch from '../commands/watch'
import Page from '../commands/page'
// import { relative } from 'path'
import neodoc from 'neodoc'

const cli = `
  🐙  Elmo – Dynamic websites on static infrastructure.

  Usage:

    elmo [options]
    elmo [options] setup [<dir>]
    elmo [options] page <page>
    elmo [options] build [<out>]
    elmo [options] serve [<port>]
    elmo [options] watch
    elmo [options] analyze

  Commands:

    setup:     scaffold a new project
    page:      scaffold a new page
    build:     build the project
    serve:     start the development server
    analyze:   print details about the build

  Global Options:

    --typescript   Setup a typescript project
    --dir <dir>    Working directory
    --dev          Development build
    --h, --help    Print this screen
    --version      Get the version
`

main().catch(err => {
  console.error(err.stack || err.message)
  process.exit(1)
})

// TODO: get more specific
type Args = {
  [key: string]: string
}

async function main() {
  const args: Args = neodoc.run(cli, { smartOptions: true, laxPlacement: true })
  const port: number = args['<port>'] ? parseInt(args['<port>'], 10) : 3000
  // const env = args['--dev'] ? 'development' : 'production'

  if (args.setup) {
    return Setup({
      root: args['<dir>'] || args['--dir'],
      typescript: !!args['--typescript']
    })
  }

  if (args.page) {
    await Page({
      root: args['--dir'],
      pageName: args['<page>'],
      typescript: !!args['--typescript']
    })
    console.error('🐙  wrote: ./%s', args['<page>'])
    return
  }

  if (args.build) {
    // TODO: support dev builds
    const files = await Build({
      root: args['--dir'],
      environment: 'production',
      outpath: args['--out']
    })
    if (files instanceof Error) {
      return process.exit(1)
    }
    return
  }

  if (args.serve) {
    // TODO: support serving production builds
    const server = await Serve({
      root: args['--dir'],
      environment: 'development'
    })
    if (server instanceof Error) {
      console.log(server)
      return process.exit(1)
    }

    server.listen(port, () => {
      console.log('listening on http://localhost:%d', port)
    })

    // const rel = relative(process.cwd(), args['--dir'])
    // if (rel) {
    //   console.error(
    //     `🐙  All set! Run \`cd ${rel} && elmo serve\` to get started.`
    //   )
    // } else {
    //   console.error(`🐙  All set! Run \`elmo serve\` to get started.`)
    // }

    return
  }

  // if (args.watch) {
  //   const webpack = await Watch({
  //     root: args['--dir'],
  //     environment: args['--dev']
  //   })
  //   if (webpack instanceof Error) {
  //     console.log(webpack)
  //     return
  //   }
  //   webpack.watch({}, function() {
  //     console.log('changed!..')
  //   })
  //   return
  // }

  if (args.analyze) {
    // TODO: support customizing the environment
    return await Analyze({
      root: args['--dir'],
      environment: 'production'
    })
  }

  console.error(cli)
  process.exit(1)
}
