//
// TODO: turn into a polyfill for:
//  https://developer.mozilla.org/en-US/docs/Web/API/URL/URL

import { parse as parseQS } from './qs'

export type URL = {
  protocol: string
  host: string
  port: number
  hostname: string
  hash: string
  search: string
  query: {
    [s: string]: string | boolean
  }
  pathname: string
  href: string
}

export function parse(url: string): URL {
  const location = window.location
  var a = document.createElement('a')
  a.href = url

  // parse the query
  const q = a.search.slice(1)
  const query = q ? parseQS(q) : {}

  return {
    protocol:
      !a.protocol || a.protocol === ':' ? location.protocol : a.protocol,
    host: a.host || location.host,
    port:
      a.port === '0' || a.port === ''
        ? port(a.protocol) || parseInt(location.port, 10)
        : parseInt(a.port, 10),
    hostname: a.hostname || location.hostname,
    hash: trimSlash(a.hash),
    search: trimSlash(a.search),
    query: query,
    pathname: trimSlash(a.pathname.charAt(0) !== '/' ? '/' + a.pathname : a.pathname),
    href: trimSlash(a.href)
  }
}

/**
 * Return default port for `protocol`.
 *
 * @param  {String} protocol
 * @return {String}
 * @api private
 */
function port(protocol: string): number {
  switch (protocol) {
    case 'http:':
      return 80
    case 'https:':
      return 443
    default:
      return 0
  }
}

/**
 * Clean path by stripping subsequent "//"'s. Without this
 * the user must be careful when to use "/" or not, which leads
 * to bad UX.
 */

function trimSlash(path: string) {
  return path === '/' ? path : path.replace(/\/+$/g, '')
}