/*
 * Module dependencies
 */

import { join, resolve, dirname } from 'path'
import * as mkdir from 'make-dir'
import pall from '../utils/pall'
import Compile from './compile'
import { File } from '../elmo'
import * as del from 'del'
import * as fs from 'mz/fs'

export type BuildOptions = {
  redirects?: { [module: string]: string }
  aliases?: { [module: string]: string }
  environment?: 'development' | 'production'
  outpath?: string
  root?: string
}

export default async function Build(
  params: BuildOptions
): Promise<File[] | Error> {
  const root = params.root ? resolve(process.cwd(), params.root) : process.cwd()
  // const force = typeof params.force !== 'undefined' ? params.force : false
  const outpath = resolve(root, params.outpath || 'site')

  // TODO: compile should return stats
  // TODO: use our messages plugin to format the output
  const files = await Compile(params)
  if (files instanceof Error) return files

  // delete the existing outpath
  await del(outpath)

  const dir = await mkdir(outpath)

  const err = await pall(
    files.map(async (f: File) => {
      const filepath = join(dir, f.path)
      await mkdir(dirname(filepath))
      fs.writeFile(filepath, f.contents, 'utf8')
    })
  )
  if (err instanceof Error) return err

  return files
}
