import { createPage, createTSPage } from '../lib/scaffolding'
import { join, dirname, relative } from 'path'
import * as mkdir from 'make-dir'
import { exists, writeFile } from 'mz/fs'

export type PageOptions = {
  typescript: boolean
  pageName: string
  root?: string
}

export default async function page(params: PageOptions) {
  const root = params.root || process.cwd()
  const isTS = await exists(join(root, "tsconfig.json"))
  const ext = isTS ? '.tsx' : '.js'
  const pages = join(root, 'pages')
  const pageName = params.pageName
  const pagePath = join(pages, pageName, pageName + ext)
  const rel = relative(process.cwd(), pagePath)



  // // TODO: parallel
  await mkdir(dirname(pagePath))
  if (await exists(pagePath)) {
    throw new Error(`${rel} already exists`)
  }

  // write the file
  if (isTS) {
    await writeFile(pagePath, createTSPage(pageName))
  } else {
    await writeFile(pagePath, createPage(pageName))
  }
}
