//

/*
 * Module dependencies
 */

// import * as WebpackDevMiddleware from 'webpack-dev-middleware'
// import * as WebpackHotMiddleware from 'webpack-hot-middleware'
import Config from '../server/config/webpack'
import MemoryFS = require('memory-fs')
import Routes from '../server/routes'
import * as Webpack from 'webpack'
import pall from '../utils/pall'
import { resolve } from 'path'

export type ServeOptions = {
  root?: string
  port?: number
  environment?: 'development' | 'production'
  aliases?: { [module: string]: string }
}

export default async function serve(
  params: ServeOptions
): Promise<Webpack.ICompiler | Error> {
  const root = params.root ? resolve(process.cwd(), params.root) : process.cwd()
  const env = params.environment || 'development'
  const aliases = params.aliases || {}
  const fs = new MemoryFS()

  // resolve routes
  const routes = await Routes(root)

  // start loading the server-side config
  const s = Config({
    environment: env,
    target: 'server',
    action: 'serve',
    routes: routes,
    aliases,
    root
  })

  // start loading the client-side config
  const b = Config({
    environment: env,
    target: 'browser',
    action: 'serve',
    routes: routes,
    aliases,
    root
  })

  const configs = await pall([s, b])
  if (configs instanceof Error) return configs

  // build the configurations in parallel
  const webpack = Webpack(configs)
  ;(webpack as any).outputFileSystem = fs

  return webpack
}
