import { h, Component, cloneElement } from 'preact'
import Router from './router'

export namespace Link {
  export interface Props {
    href: string
  }

  export interface Context {
    router: Router
  }
}

export default class Link extends Component<Link.Props, never> {
  constructor(props: Link.Props, context: Link.Context) {
    super(props, context)
  }

  onClick = (e: MouseEvent) => {
    if (!this.context.router) {
      throw new Error('<Link /> no router passed in through context')
    }

    e.preventDefault()
    this.context.router.push(this.props.href)
  }

  render() {
    const { href, children } = this.props
    if (!children || !children[0]) return null
    const firstChild = children[0]

    if (
      !href ||
      typeof firstChild === 'string' ||
      !firstChild.hasOwnProperty('nodeName')
    ) {
      return (
        <a href={href} onClick={this.onClick}>
          {children}
        </a>
      )
    }

    return cloneElement(firstChild, { onClick: this.onClick })
  }
}
