import { Component } from 'preact'
import render, { Upgrader } from './render'

export interface Props {
  html?: string
  xml?: string
  upgrade: Upgrader
}

export default class Markup extends Component<Props, never> {
  private upgrader: Upgrader

  shouldComponentUpdate({ html, xml }: Props) {
    let p = this.props
    return html !== p.html || xml !== p.xml
  }

  setComponents(components: Upgrader) {
    this.upgrader = {}
    if (components) {
      for (let i in components) {
        if (components.hasOwnProperty(i)) {
          let name = i
            .replace(/([A-Z]+)([A-Z][a-z0-9])|([a-z0-9]+)([A-Z])/g, '$1$3-$2$4')
            .toLowerCase()
          this.upgrader[name] = components[i]
        }
      }
    }
  }

  render(props: Props) {
    if (!props.html && !props.xml) return null
    this.setComponents(props.upgrade)
    const type = props.html ? 'html' : 'xml'
    const vdom = render(props.html || props.xml, props.upgrade, { type: type })
    return vdom && typeof vdom !== 'string' && vdom.length ? vdom[0] : null
  }
}
