/**
 * Module dependencies
 */

import compile from './compile'
import parse from './parse'

/**
 * Map
 */

export type Props = any
export type Upgrader = { [tag: string]: (c: Props) => JSX.Element | null }

/**
 * Options
 */

export type Options = {
  type: 'html' | 'xml'
}

/**
 * HTML function
 */

export default function Render(
  html: string = '',
  upgrader?: Upgrader,
  options: Options = { type: 'html' }
): JSX.Element[] | string | null {
  const dom = parse(html, options.type)
  if (dom && dom.error) throw new Error(dom.error)
  const body = (dom && dom.body) || dom

  const vdom = compile(body, upgrader, options)
  if (vdom === null) return null
  if (typeof vdom === 'string') return vdom
  return vdom && vdom.children
}
