import { Component } from 'preact'
import render from './render'

export type Props = {
  id: string
  css: string
}

type Updated = {
  instance: Component<Props, Object>
  id: string
  css: string
}

export type Styles = {
  [id: string]: string
}

let components: Component<Props, Object>[] = []

export default class Style extends Component<Props, {}> {
  componentWillMount() {
    components.push(this)
    update()
  }

  // To avoid FOUC, we process new changes
  // on `componentWillUpdate` rather than `componentDidUpdate`.
  componentWillUpdate(nextProps: Props) {
    update({
      instance: this,
      id: nextProps.id,
      css: nextProps.css
    })
  }

  componentWillUnmount() {
    const i = components.indexOf(this)
    if (i < 0) {
      return
    }

    update()
    components.splice(i, 1)
  }

  render() {
    return null
  }

  static flush(): Styles {
    const ret = stylesMap()
    components = []
    return ret
  }

  // static default = Style
}
// module.exports = Style

function update(updated?: Updated): void {
  render(stylesMap(updated))
  // console.log('rendering', stylesMap(updated))
  // console.log(stylesMap(updated))
}

function stylesMap(updated?: Updated): Styles {
  const ret: Styles = {}
  for (const c of components) {
    if (updated && c === updated.instance) {
      // On `componentWillUpdate`
      // we use `styleId` and `css` from updated
      // component rather than reading `props`
      // from the component since they haven't
      // been updated yet.
      ret[updated.id] = updated.css
    } else {
      ret[c.props.id] = c.props.css
    }
  }
  return ret
}
