/**
 * Dependencies
 */

// import parseHTML from 'rehype-parser'
// import Transform from '../../lib/style/transform'
// import { validate } from '../config/tsconfig'
import { JS, HTMLElement } from '../../lib/mdx'
import * as toSlug from 'to-slug-case'
import { join, sep } from 'path'
import { loader } from 'webpack'
// import * as ts from 'typescript'

/**
 * Export the `loader`
 */

export default function(
  this: loader.LoaderContext,
  source: string,
  _sourceMap: string
) {
  this.cacheable()

  const { resourcePath, query: { root } } = this
  const resolve = (node: HTMLElement) => resolver(root, node)

  // compile MDX
  let result = ''
  try {
    result = JS(source, { resolve })
  } catch (err) {
    err.stack =
      `mdx loader: error loading '${resourcePath}'\n\n` + (err.stack || '')
    this.callback(err)
    return
  }

  this.callback(null, result, undefined)
}

// Opinionated Resolver for Elmo:
//
// <Link> => ./components/link or ./components/link/index
//
// <h1>   => ./components/h1 or ./components/h1/index
//
// <Gallery:Video> => ./components/gallery/video
//                 or ./components/gallery/video/index
//
function resolver(root: string, node: HTMLElement): string {
  const p = './' + toSlug(node.tagName.replace(':', sep))
  return requires(
    join(root, 'components', p + '.tsx'),
    join(root, 'components', p + '.jsx'),
    join(root, 'components', p, 'index.tsx'),
    join(root, 'components', p, 'index.jsx')
  )
}

/**
 * Requires
 */
function requires(...paths: string[]): string {
  for (let i = 0; i < paths.length; i++) {
    try {
      return require.resolve(paths[i])
    } catch (err) {}
  }
  return ''
}
