import { Route } from '../../elmo'
import { join, basename, resolve } from 'path'
import RoutePath from '../../lib/route'
import * as Regexp from 'path-to-regexp'
import * as globby from 'globby'
import { exists } from 'mz/fs'

/**
 * Providers
 */

import Redirects from './redirects'
import Package from './package'

/**
 * Priority array
 */

const priority = ['_redirects', 'package.json']

type StringMap = {
  [s: string]: string
}

/**
 * Routing
 */

export default async function Routing(root: string): Promise<Route[]> {
  let pages = await globby(join(root, 'pages', '**', '*.{js,jsx,ts,tsx}'))
  pages = pages.filter(page => basename(page)[0] !== '_')
  const routes = await find(root)

  const pagemap: StringMap = pages.reduce((routes: StringMap, p) => {
    let r = RoutePath(root, p)
    routes[r] = r
    return routes
  }, {})

  const out: Route[] = []

  for (let k in routes) {
    out.push(compile(k, routes[k]))
  }

  for (let k in pagemap) {
    out.push(compile(k, pagemap[k]))
  }

  return out
}

async function find(root: string): Promise<StringMap> {
  const paths = priority.map(p => resolve(root, p))
  const existing = await Promise.all(paths.map(async p => exists(p)))
  for (let i = 0, len = existing.length; i < len; i++) {
    if (!existing[i]) continue
    switch (priority[i]) {
      case '_redirects':
        return Redirects(paths[i])
      case 'package.json':
        return Package(paths[i])
    }
  }
  return {}
}

function compile(expr: string, path: string): Route {
  const params: Regexp.Key[] = []
  const regexp = Regexp(expr, params)
  return {
    path: path,
    regexp: regexp.source,
    params: params.map(p => p.name)
  }
}
